@@ -26,7 +26,7 @@ class Rake::TestCase < Minitest::Test
     include Rake::TaskManager
   end
 
-  RUBY = ENV['BUNDLE_RUBY'] || Gem.ruby
+  RUBY = ENV["BUNDLE_RUBY"] || Gem.ruby
 
   def setup
     ARGV.clear
