@@ -1,4 +1,4 @@
-#encoding: UTF-8
+# frozen_string_literal: true
 require File.expand_path("../helper", __FILE__)
 
 class TestRakeApplication < Rake::TestCase
@@ -10,11 +10,34 @@ class TestRakeApplication < Rake::TestCase
     @app.options.rakelib = []
   end
 
-  def setup_command_line(*options)
-    ARGV.clear
-    options.each do |option|
-      ARGV << option
+  def test_class_with_application
+    orig_app = Rake.application
+
+    return_app = Rake.with_application do |yield_app|
+      refute_equal orig_app, yield_app, "new application must be yielded"
+
+      assert_equal yield_app, Rake.application,
+                   "new application must be default in block"
     end
+
+    refute_equal orig_app, return_app, "new application not returned"
+    assert_equal orig_app, Rake.application, "original application not default"
+  end
+
+  def test_class_with_application_user_defined
+    orig_app = Rake.application
+
+    user_app = Rake::Application.new
+
+    return_app = Rake.with_application user_app do |yield_app|
+      assert_equal user_app, yield_app, "user application must be yielded"
+
+      assert_equal user_app, Rake.application,
+                   "user application must be default in block"
+    end
+
+    assert_equal user_app, return_app, "user application not returned"
+    assert_equal orig_app, Rake.application, "original application not default"
   end
 
   def test_display_exception_details
@@ -26,6 +49,8 @@ class TestRakeApplication < Rake::TestCase
     end
 
     out, err = capture_io do
+      @app.set_default_options # reset trace output IO
+
       @app.display_error_message ex
     end
 
@@ -37,11 +62,13 @@ class TestRakeApplication < Rake::TestCase
 
   def test_display_exception_details_bad_encoding
     begin
-      raise "El Niño is coming!".force_encoding("US-ASCII")
+      raise "El Niño is coming!".dup.force_encoding("US-ASCII")
     rescue => ex
     end
 
     out, err = capture_io do
+      @app.set_default_options # reset trace output IO
+
       @app.display_error_message ex
     end
 
@@ -63,39 +90,14 @@ class TestRakeApplication < Rake::TestCase
     end
 
     out, err = capture_io do
+      @app.set_default_options # reset trace output IO
+
       @app.display_error_message ex
     end
 
     assert_empty out
 
-    assert_match "cause a", err
-    assert_match "cause b", err
-  end
-
-  def test_display_exception_details_cause_loop
-    skip "Exception#cause not implemented" unless
-      Exception.method_defined? :cause
-    skip if jruby9? # https://github.com/jruby/jruby/issues/3654
-
-    begin
-      begin
-        raise "cause a"
-      rescue => a
-        begin
-          raise "cause b"
-        rescue
-          raise a
-        end
-      end
-    rescue => ex
-    end
-
-    out, err = capture_io do
-      @app.display_error_message ex
-    end
-
-    assert_empty out
-
+    assert_match "Caused by:", err
     assert_match "cause a", err
     assert_match "cause b", err
   end
@@ -215,7 +217,7 @@ class TestRakeApplication < Rake::TestCase
     rakefile_unittest
 
     @app.instance_eval do
-      handle_options
+      handle_options []
       options.silent = true
       load_rakefile
     end
@@ -243,7 +245,7 @@ class TestRakeApplication < Rake::TestCase
     Dir.chdir "subdir"
 
     @app.instance_eval do
-      handle_options
+      handle_options []
       options.silent = true
       load_rakefile
     end
@@ -274,7 +276,7 @@ class TestRakeApplication < Rake::TestCase
 
     _, err = capture_io do
       @app.instance_eval do
-        handle_options
+        handle_options []
         options.silent = true
         raw_load_rakefile
       end
@@ -286,12 +288,11 @@ class TestRakeApplication < Rake::TestCase
   def test_load_rakefile_not_found
     skip if jruby9?
 
-    ARGV.clear
     Dir.chdir @tempdir
     ENV["RAKE_SYSTEM"] = "not_exist"
 
     @app.instance_eval do
-      handle_options
+      handle_options []
       options.silent = true
     end
 
@@ -308,7 +309,7 @@ class TestRakeApplication < Rake::TestCase
     rake_system_dir
 
     @app.instance_eval do
-      handle_options
+      handle_options []
       options.silent = true
       options.load_system = true
       options.rakelib = []
@@ -330,7 +331,7 @@ class TestRakeApplication < Rake::TestCase
     ENV["RAKE_SYSTEM"] = nil
 
     @app.instance_eval do
-      handle_options
+      handle_options []
       options.silent = true
       options.load_system = true
       options.rakelib = []
@@ -390,28 +391,22 @@ class TestRakeApplication < Rake::TestCase
   def test_handle_options_should_not_strip_options_from_argv
     assert !@app.options.trace
 
-    valid_option = "--trace"
-    setup_command_line(valid_option)
+    argv = %w[--trace]
+    @app.handle_options argv
 
-    @app.handle_options
-
-    assert ARGV.include?(valid_option)
+    assert_includes argv, "--trace"
     assert @app.options.trace
   end
 
   def test_handle_options_trace_default_is_stderr
-    setup_command_line("--trace")
-
-    @app.handle_options
+    @app.handle_options %w[--trace]
 
     assert_equal STDERR, @app.options.trace_output
     assert @app.options.trace
   end
 
   def test_handle_options_trace_overrides_to_stdout
-    setup_command_line("--trace=stdout")
-
-    @app.handle_options
+    @app.handle_options %w[--trace=stdout]
 
     assert_equal STDOUT, @app.options.trace_output
     assert @app.options.trace
@@ -420,18 +415,16 @@ class TestRakeApplication < Rake::TestCase
   def test_handle_options_trace_does_not_eat_following_task_names
     assert !@app.options.trace
 
-    setup_command_line("--trace", "sometask")
+    argv = %w[--trace sometask]
+    @app.handle_options argv
 
-    @app.handle_options
-    assert ARGV.include?("sometask")
+    assert argv.include?("sometask")
     assert @app.options.trace
   end
 
   def test_good_run
     ran = false
 
-    ARGV << '--rakelib=""'
-
     @app.options.silent = true
 
     @app.instance_eval do
@@ -441,7 +434,7 @@ class TestRakeApplication < Rake::TestCase
     rakefile_default
 
     out, err = capture_io do
-      @app.run
+      @app.run %w[--rakelib=""]
     end
 
     assert ran
@@ -451,10 +444,9 @@ class TestRakeApplication < Rake::TestCase
 
   def test_display_task_run
     ran = false
-    setup_command_line("-f", "-s", "--tasks", '--rakelib=""')
     @app.last_description = "COMMENT"
     @app.define_task(Rake::Task, "default")
-    out, = capture_io { @app.run }
+    out, = capture_io { @app.run %w[-f -s --tasks --rakelib=""] }
     assert @app.options.show_tasks
     assert ! ran
     assert_match(/rake default/, out)
@@ -463,13 +455,12 @@ class TestRakeApplication < Rake::TestCase
 
   def test_display_prereqs
     ran = false
-    setup_command_line("-f", "-s", "--prereqs", '--rakelib=""')
     @app.last_description = "COMMENT"
     t = @app.define_task(Rake::Task, "default")
     t.enhance([:a, :b])
     @app.define_task(Rake::Task, "a")
     @app.define_task(Rake::Task, "b")
-    out, = capture_io { @app.run }
+    out, = capture_io { @app.run %w[-f -s --prereqs --rakelib=""] }
     assert @app.options.show_prereqs
     assert ! ran
     assert_match(/rake a$/, out)
@@ -479,37 +470,29 @@ class TestRakeApplication < Rake::TestCase
 
   def test_bad_run
     @app.intern(Rake::Task, "default").enhance { fail }
-    setup_command_line("-f", "-s",  '--rakelib=""')
     _, err = capture_io {
-      assert_raises(SystemExit) { @app.run }
+      assert_raises(SystemExit) { @app.run %w[-f -s --rakelib=""] }
     }
     assert_match(/see full trace/i, err)
-  ensure
-    ARGV.clear
   end
 
   def test_bad_run_with_trace
     @app.intern(Rake::Task, "default").enhance { fail }
-    setup_command_line("-f", "-s", "-t")
     _, err = capture_io {
-      assert_raises(SystemExit) { @app.run }
+      @app.set_default_options
+      assert_raises(SystemExit) { @app.run %w[-f -s -t] }
     }
     refute_match(/see full trace/i, err)
-  ensure
-    ARGV.clear
   end
 
   def test_bad_run_with_backtrace
     @app.intern(Rake::Task, "default").enhance { fail }
-    setup_command_line("-f", "-s", "--backtrace")
     _, err = capture_io {
       assert_raises(SystemExit) {
-        @app.run
+        @app.run %w[-f -s --backtrace]
       }
     }
     refute_match(/see full trace/, err)
-  ensure
-    ARGV.clear
   end
 
   CustomError = Class.new(RuntimeError)
@@ -518,10 +501,9 @@ class TestRakeApplication < Rake::TestCase
     @app.intern(Rake::Task, "default").enhance {
       raise CustomError, "intentional"
     }
-    setup_command_line("-f", "-s")
     _, err = capture_io {
       assert_raises(SystemExit) {
-        @app.run
+        @app.run %w[-f -s]
       }
     }
     assert_match(/CustomError: intentional/, err)
@@ -531,10 +513,9 @@ class TestRakeApplication < Rake::TestCase
     @app.intern(Rake::Task, "default").enhance {
       fail "intentional"
     }
-    setup_command_line("-f", "-s")
     _, err = capture_io {
       assert_raises(SystemExit) {
-        @app.run
+        @app.run %w[-f -s]
       }
     }
    refute_match(/RuntimeError/, err)
@@ -555,28 +536,22 @@ class TestRakeApplication < Rake::TestCase
         raise custom_error, "Secondary Error"
       end
     }
-    setup_command_line("-f", "-s")
     _ ,err = capture_io {
       assert_raises(SystemExit) {
-        @app.run
+        @app.run %w[-f -s]
       }
     }
     if cause_supported?
       assert_match(/Original Error/, err)
     end
     assert_match(/Secondary Error/, err)
-  ensure
-    ARGV.clear
   end
 
   def test_run_with_bad_options
     @app.intern(Rake::Task, "default").enhance { fail }
-    setup_command_line("-f", "-s", "--xyzzy")
     assert_raises(SystemExit) {
-      capture_io { @app.run }
+      capture_io { @app.run %w[-f -s --xyzzy] }
     }
-  ensure
-    ARGV.clear
   end
 
   def test_standard_exception_handling_invalid_option
@@ -596,6 +571,8 @@ class TestRakeApplication < Rake::TestCase
 
   def test_standard_exception_handling_other
     out, err = capture_io do
+      @app.set_default_options # reset trace output IO
+
       e = assert_raises SystemExit do
         @app.standard_exception_handling do
           raise "blah"
