@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require File.expand_path("../helper", __FILE__)
 
 TESTING_REQUIRE = []
@@ -8,7 +9,6 @@ class TestRakeApplicationOptions < Rake::TestCase
     super
 
     @testkey = ENV["TESTKEY"]
-    clear_argv
     Rake::FileUtilsExt.verbose_flag = false
     Rake::FileUtilsExt.nowrite_flag = false
     TESTING_REQUIRE.clear
@@ -16,32 +16,27 @@ class TestRakeApplicationOptions < Rake::TestCase
 
   def teardown
     ENV["TESTKEY"] = @testkey
-    clear_argv
     Rake::FileUtilsExt.verbose_flag = false
     Rake::FileUtilsExt.nowrite_flag = false
 
     super
   end
 
-  def clear_argv
-    ARGV.pop until ARGV.empty?
-  end
-
   def test_default_options
     opts = command_line
-    assert_nil opts.backtrace
-    assert_nil opts.dryrun
-    assert_nil opts.ignore_system
-    assert_nil opts.load_system
-    assert_nil opts.always_multitask
-    assert_nil opts.nosearch
+    assert_equal false, opts.backtrace
+    assert_equal false, opts.dryrun
+    assert_equal false, opts.ignore_system
+    assert_equal false, opts.load_system
+    assert_equal false, opts.always_multitask
+    assert_equal false, opts.nosearch
     assert_equal ["rakelib"], opts.rakelib
-    assert_nil opts.show_prereqs
+    assert_equal false, opts.show_prereqs
     assert_nil opts.show_task_pattern
     assert_nil opts.show_tasks
-    assert_nil opts.silent
-    assert_nil opts.trace
-    assert_nil opts.thread_pool_size
+    assert_equal false, opts.silent
+    assert_equal false, opts.trace
+    assert_equal Rake.suggested_thread_count, opts.thread_pool_size
     assert_equal ["rakelib"], opts.rakelib
     assert ! Rake::FileUtilsExt.verbose_flag
     assert ! Rake::FileUtilsExt.nowrite_flag
@@ -114,7 +109,7 @@ class TestRakeApplicationOptions < Rake::TestCase
 
   def test_jobs
     flags([]) do |opts|
-      assert_nil opts.thread_pool_size
+      assert_equal Rake.suggested_thread_count, opts.thread_pool_size
     end
     flags(["--jobs", "0"], ["-j", "0"]) do |opts|
       assert_equal 0, opts.thread_pool_size
@@ -328,12 +323,12 @@ class TestRakeApplicationOptions < Rake::TestCase
     flags("--tasks", "-T") do |opts|
       assert_equal :tasks, opts.show_tasks
       assert_equal(//.to_s, opts.show_task_pattern.to_s)
-      assert_nil opts.show_all_tasks
+      assert_equal false, opts.show_all_tasks
     end
     flags(["--tasks", "xyz"], ["-Txyz"]) do |opts|
       assert_equal :tasks, opts.show_tasks
       assert_equal(/xyz/.to_s, opts.show_task_pattern.to_s)
-      assert_nil opts.show_all_tasks
+      assert_equal false, opts.show_all_tasks
     end
     flags(["--tasks", "xyz", "--comments"]) do |opts|
       assert_equal :tasks, opts.show_tasks
@@ -444,8 +439,6 @@ class TestRakeApplicationOptions < Rake::TestCase
 
   def flags(*sets)
     sets.each do |set|
-      ARGV.clear
-
       @exit = catch(:system_exit) { command_line(*set) }
 
       yield(@app.options) if block_given?
@@ -453,13 +446,12 @@ class TestRakeApplicationOptions < Rake::TestCase
   end
 
   def command_line(*options)
-    options.each do |opt| ARGV << opt end
     @app = Rake::Application.new
     def @app.exit(*args)
       throw :system_exit, :exit
     end
     @app.instance_eval do
-      args = handle_options
+      args = handle_options options
       collect_command_line_tasks(args)
     end
     @tasks = @app.top_level_tasks
