@@ -1,8 +1,9 @@
+# frozen_string_literal: true
 require File.expand_path("../helper", __FILE__)
 
 class TestRakePathMapPartial < Rake::TestCase
   def test_pathmap_partial
-    @path = "1/2/file"
+    @path = "1/2/file".dup
     def @path.call(n)
       pathmap_partial(n)
     end
