@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require File.expand_path("../helper", __FILE__)
 require "fileutils"
 
@@ -61,10 +62,14 @@ class TestRakeTask < Rake::TestCase
   end
 
   def test_dry_run_prevents_actions
-    Rake.application.options.dryrun = true
     runlist = []
     t1 = task(:t1) { |t| runlist << t.name; 3321 }
-    _, err = capture_io { t1.invoke }
+    _, err = capture_io {
+      Rake.application.set_default_options # reset trace output IO
+      Rake.application.options.dryrun = true
+
+      t1.invoke
+    }
     assert_match(/execute .*t1/i, err)
     assert_match(/dry run/i, err)
     refute_match(/invoke/i, err)
@@ -74,9 +79,11 @@ class TestRakeTask < Rake::TestCase
   end
 
   def test_tasks_can_be_traced
-    Rake.application.options.trace = true
     t1 = task(:t1)
     _, err = capture_io {
+      Rake.application.set_default_options # reset trace output IO
+      Rake.application.options.trace = true
+
       t1.invoke
     }
     assert_match(/invoke t1/i, err)
@@ -452,4 +459,15 @@ class TestRakeTask < Rake::TestCase
     t = task t: ["preqA", "preqB"]
     assert_equal "preqA", t.source
   end
+
+  def test_suggests_valid_rake_task_names
+    task :test
+    error = assert_raises(RuntimeError) { Task[:testt] }
+
+    assert_match(/Don\'t know how to build task \'testt\'/, error.message)
+
+    if defined?(::DidYouMean::SpellChecker) && defined?(::DidYouMean::Formatter)
+      assert_match(/Did you mean\?  test/, error.message)
+    end
+  end
 end
