@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require File.expand_path("../helper", __FILE__)
 require "stringio"
 
@@ -8,7 +9,7 @@ class TestTraceOutput < Rake::TestCase
     attr_reader :result, :calls
 
     def initialize
-      @result = ""
+      @result = "".dup
       @calls = 0
     end
 
